@@ -43,7 +43,17 @@ def eigh_projector(
     if rank > 32767:
         raise RuntimeError("Projector rank is too large in eigh.")
 
-    eigvals, eigvecs = np.linalg.eigh(p)
+    try:
+        eigvals, eigvecs = np.linalg.eigh(p)
+    except np.linalg.LinAlgError as e:
+        if verbose:
+            print(f"np.linalg.eigh failed: {str(e)}")
+            print("Try scipy.linalg.lapack.dsyev")
+        eigvals, eigvecs, info = scipy.linalg.lapack.dsyev(p.T)
+        if info != 0:
+            raise scipy.linalg.LinAlgError(
+                "scipy.linalg.lapack.dsyev failed: Eigenvalues did not converge"
+            ) from e
 
     tol = 1e-8
     if np.count_nonzero((eigvals > 1.0 + tol) | (eigvals < -tol)):
